#include <ibsys.h>


/*
 * There is a Problem with Linux Semaphores: 
 *      - enabling Interrupts
 *      - call to down()
 *
 * could produce spurious segmentation faults, because up() is called
 * within this interrupt just before down() has been called
 * then wait will be called with empty queue.
 *
 * I worked over this code again to protect from such conditions
 * hope it works. :-)
 *
 *
 *
 */

extern uint8 ibirq;

#if USEINTS
/*
 * GPIB interrupt service routine -- fast and simple
 */
IBLCL void ibintr(int irq)
{
int s;
        printk("GPIB-IRQ \n");

#if DEBUG
	if (dbgMask & DBG_INTR)
#if HAS_SEMAPHORES
	        printk("GPIB INTERRUPT! semaphore id = %d\n", espsemid.count);
#else
		printk("GPIB INTERRUPT!\n");
#endif
#endif
#ifdef NIAT
	GPIBout(imr3, 0);		                /* disable interrupts */
#endif
#ifdef NIPCII
	GPIBout(imr2, 0);
#endif
#ifdef NIPCIIa

	GPIBout(imr2, 0);
        /* reset shared interrupt circuit (Ed Clarke told me) */
        /*osSendEOI();*/
        printk("GPIB-IRQ nipcii\n");
	outb(0xff , ( 0x2f0 | ibirq ) );

#endif
#if HAS_SEMAPHORES
/*printk("semid=%d ",espsemid.count);*/
	    up(&espsemid); /* wake up sleeping process */
#else
        espsemid--;
	wake_up(&ibwait_queue);		/* wake up sleeping task */
#endif

}


/*
 * Wait for GPIB or timer interrupt.  Semaphore will be posted by
 * either ibintr or ibtmintr.
 */



IBLCL osWaitForInt(imr3mask)
int imr3mask;
{
        struct wait_queue wait = {current, NULL}; /*@*/

	DBGin("osWaitForInt");
#if HAS_SEMAPHORES
	if (espsemid.count <= 0) {
#else
	if (espsemid == 0) {
#endif
	/*
	 *	If semaphore not already available, enable
	 *	requested interrupts and wait until it is...
	 */

	  espsemid.count = 1;
          down(&espsemid); /* spurious interrups calling up() while enable irq's ? */

          /* now it's time to enable board interrupts */
#ifdef NIAT    
	        DBGprint(DBG_DATA, ("imr3mask=0x%x  ", imr3mask));		
                GPIBout(imr3, imr3mask);
#endif
#ifdef NIPCII
                DBGprint(DBG_DATA, ("imr2mask=0x%x  ", imr3mask));		
		GPIBout(imr2, imr3mask); /* sorry */
	        
#endif
#if HAS_SEMAPHORES

		/* now push process to sleep */
		down(&espsemid);
		up(&espsemid); /* wakeup process */

#endif

#if !HAS_SEMAPHORES
		espsemid++;
		interruptible_sleep_on(&ibwait_queue);
#endif

	}
#if HAS_SEMAPHORES
        else	{ 
	  espsemid.count = 0;
          espsemid.wait  = NULL;
	}
#else
	else	espsemid = 0;
#endif

	DBGout();
}
#endif














