/*
 *
 * ibDll.c -
 *     This Module Implements dynymic loading Commands for the Tcl/Tk Toolkit
 *     Using GNU - dld Library Routines.
 *
 *     (c) 1994 C. Schroeter (clausi@chemie.fu-berlin.de)
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include <tcl.h>
#include <dld.h>


Tcl_DString Dll_suffix;
Tcl_DString Dll_path;
Tcl_DString Dll_cmd;


#ifndef DLL_DEFAULT_SUFFIX
#define DLL_DEFAULT_SUFFIX ".o"
#endif

#ifndef DLL_DEFAULT_PATH
#define DLL_DEFAULT_PATH   "/usr/local/lib/tcl/modules"
#endif



int Dll_Init ( Tcl_Interp *interp ){

extern int DllCmd _ANSI_ARGS_(( ClientData clientData,
			      Tcl_Interp *interp,
			       int argc,
			       char *argv[]
			       ));

    Tcl_CreateCommand(interp,"dll",DllCmd,
		         (ClientData) NULL,
			 (Tcl_CmdDeleteProc *) NULL );


    Tcl_DStringInit(&Dll_suffix);
    Tcl_DStringInit(&Dll_path);

    Tcl_DStringAppend(&Dll_suffix, DLL_DEFAULT_SUFFIX, -1 );
    Tcl_DStringAppend(&Dll_path,DLL_DEFAULT_PATH, -1);

    return TCL_OK;
}


/**********************************************************************/






int DllCmd _ANSI_ARGS_(( ClientData clientData,
			Tcl_Interp *interp,
			int argc,
			char *argv[]
			)) 
{
 register i;
 int (*init_func)    ( Tcl_Interp *interp );
 int (*cleanup_func) ( Tcl_Interp *interp );
			

 Tcl_DString modname;
 char  func[30];
 char **undefs;
 




 /* LOAD ***************************************************************/

 if ( *argv[1] == 'l' && !strcmp(argv[1],"load")) {


   if( argc < 3 ){
     Tcl_AppendResult(interp,"Usage: dll load <module> [path]",(char *) NULL);
     return TCL_ERROR;
   }


   Tcl_DStringInit( &modname );

   if( argc > 3 ){		                                /* load path specified */
     if ( *argv[3] == '~' ) 
       Tcl_TildeSubst( interp, argv[3], &modname );
     else
       Tcl_DStringAppend( &modname, argv[3], -1 );
     Tcl_DStringAppend( &modname, "/", -1 );
   } else {           
                                          /* try default load path */
     if ( *argv[3] == '~' ) 
       Tcl_TildeSubst( interp, Dll_path.string , &modname );
     else
       Tcl_DStringAppend( &modname, Dll_path.string , -1 );
     Tcl_DStringAppend( &modname, "/", -1 );
   }

     Tcl_DStringAppend( &modname, argv[2] , -1 );               /* add name */
     Tcl_DStringAppend( &modname, Dll_suffix.string , -1 );     /* add suffix */



   if( Dll_ModulePresent(interp,argv[2])){
     Tcl_AppendResult(interp,"Module ",argv[2]," already loaded.",(char *) NULL);
     Tcl_DStringFree(&modname);
     return TCL_ERROR;
   }


   if( dld_init( dld_find_executable( Tcl_GetVar(interp, "argv00", TCL_GLOBAL_ONLY) ) ) ){
     Tcl_AppendResult(interp,"Dll: Problem finding main process:", 
		      Tcl_GetVar(interp, "argv00", TCL_GLOBAL_ONLY) ,(char *) NULL);
     Tcl_DStringFree(&modname);
     return TCL_ERROR;
   }

   strcpy(func,argv[2]);
   strcat(func,"_init");

   if( dld_link(modname.string) ){
     dld_perror("gnurps");
     Tcl_AppendResult(interp,"Dll: Link Error: can't find ",modname.string ,(char *) NULL);
     Tcl_DStringFree(&modname);
     return TCL_ERROR;
   }

   Tcl_DStringFree(&modname);


   if(( init_func = (int (*)()) dld_get_func(func)) != 0 ){
     if( dld_function_executable_p(func)){
       (*init_func)( interp );
     } else {
       Tcl_AppendResult(interp,"Dll: Undefined symbols:\n",(char *) NULL);
       undefs=dld_list_undefined_sym ();
       for(i=0;i<dld_undefined_sym_count;i++)
	 Tcl_AppendResult(interp,undefs[i],"\n",(char *) NULL);
       return TCL_ERROR;
     }
     Dll_registerModule(interp, argv[2]);
     return TCL_OK;
   } else {
     Tcl_AppendResult(interp,"DLL: Module has no function ",func,(char *) NULL);
     return TCL_ERROR;
   }


 }

 /* UNLOAD ***************************************************************/

 if ( *argv[1] == 'u' && !strcmp(argv[1],"unload")) {


   if( argc != 3 ){
     Tcl_AppendResult(interp,"Usage: dll unload <module> ",(char *) NULL);
     return TCL_ERROR;
   }

   if( ! Dll_ModulePresent(interp,argv[2])){
     Tcl_AppendResult(interp,"Dll: Module ",argv[2]," not present.",(char *) NULL);
     return TCL_ERROR;
   }

   strcpy(func,argv[2]);
   strcat(func,"_cleanup");

   if(( cleanup_func = (int (*)()) dld_get_func(func)) != 0 ){
     if( dld_function_executable_p(func)){
       (*cleanup_func)( interp );                    /* call module cleanup func */
     } else {
       Tcl_AppendResult(interp,"Dll: Undefined symbols:\n",(char *) NULL);
       undefs=dld_list_undefined_sym ();
       for(i=0;i<dld_undefined_sym_count;i++)
       Tcl_AppendResult(interp,undefs[i],"\n",(char *) NULL);
       return TCL_ERROR;
     }
     /* remove module from interpreter */
     if( dld_unlink_by_symbol(func,0) ){
       Tcl_AppendResult(interp,"Dll: Can't unlink",func,(char *) NULL);
       return TCL_ERROR;
     }
     Dll_unregisterModule(interp, argv[2]);
     return TCL_OK;
   } else {
     Tcl_AppendResult(interp,"DLL: Module has no function ",func,(char *) NULL);
     return TCL_ERROR;
   }
 }

 /* SET ***************************************************************/

 if ( *argv[1] == 's' && !strcmp(argv[1],"set")) {

   if( argc < 4 ){
     Tcl_AppendResult(interp,"Usage: dll set <parameter> NewValue ",(char *) NULL);
     return TCL_ERROR;
   }

   if ( *argv[2]=='s' && !strcmp(argv[2],"suffix")){ 
     Tcl_DStringFree(&Dll_suffix);
     Tcl_DStringInit(&Dll_suffix);
     Tcl_DStringAppend(&Dll_suffix, argv[3], -1);
     return TCL_OK;
   }

   if ( *argv[2]=='p' && !strcmp(argv[2],"path")){ 
     Tcl_DStringFree(&Dll_path);
     Tcl_DStringInit(&Dll_path);
     Tcl_DStringAppend(&Dll_path, argv[3], -1);
     return TCL_OK;
   }

   Tcl_AppendResult(interp,"Dll: dll set : illegal parameter ",(char *) NULL);
   return TCL_ERROR;
   

 }

 Tcl_AppendResult(interp,"Usage: dll command <parameter> ",(char *) NULL);
 return TCL_ERROR;
 

}

/***********************************************************************/
int Dll_registerModule _ANSI_ARGS_((Tcl_Interp *interp, char *func)){

if( Tcl_SetVar2(interp, "dll_module", func , "1", TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG ) == NULL )
  return TCL_ERROR;
else
  return TCL_OK;

}


int Dll_unregisterModule _ANSI_ARGS_((Tcl_Interp *interp,char *func)){

return Tcl_UnsetVar2(interp, "dll_module", func , TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG);

}

int Dll_ModulePresent _ANSI_ARGS_((Tcl_Interp *interp, char *func)){

if( Tcl_GetVar2(interp,"dll_module", func, TCL_GLOBAL_ONLY ) == NULL )
  return 0;
else
  return 1;

}















